package org.openjdk.gcbench.tests;

import org.openjdk.jmh.runner.options.Options;
import org.openjdk.jmh.util.Utils;

import java.io.PrintWriter;
import java.util.concurrent.TimeUnit;

public abstract class AbstractTest implements Test {

    protected final PrintWriter pw;
    protected final Options baseOpts;
    public final Class<?> benchmark;
    protected final String label;
    protected final String description;

    public AbstractTest(Options baseOpts, String label, Class<?> benchmark, String description) {
        this.baseOpts = baseOpts;
        this.label = label;
        this.benchmark = benchmark;
        this.description = description;
        this.pw = new PrintWriter(System.out, true);
    }

    @Override
    public String label() {
        return label;
    }

    @Override
    public String description() {
        return description;
    }

    @Override
    public final void run() {
        pw.println("===== " + label);
        pw.println();

        Utils.reflow(pw,
                description,
                80, 2);

        pw.println();

        long time1 = System.nanoTime();

        doRun();

        long time2 = System.nanoTime();

        pw.println();
        pw.println("  Test took " + TimeUnit.NANOSECONDS.toSeconds(time2 - time1) + " seconds.");
        pw.println();
    }

    protected abstract void doRun();
}
